@@ -31,7 +31,7 @@ from pipecat.frames.frames import (
 )
 from pipecat.processors.frame_processor import FrameDirection
 from pipecat.services.stt_service import STTService
-from pipecat.transcriptions.language import Language
+from pipecat.transcriptions.language import Language, resolve_language
 from pipecat.utils.tracing.service_decorators import traced_stt
 
 try:
@@ -577,6 +577,7 @@ class SpeechmaticsSTTService(STTService):
                 ),
             )
             logger.debug(f"{self} Connected to Speechmatics STT service")
+            await self._call_event_handler("on_connected")
         except Exception as e:
             logger.error(f"{self} Error connecting to Speechmatics: {e}")
             self._client = None
@@ -595,6 +596,7 @@ class SpeechmaticsSTTService(STTService):
             logger.error(f"{self} Error closing Speechmatics client: {e}")
         finally:
             self._client = None
+            await self._call_event_handler("on_disconnected")
 
     def _process_config(self) -> None:
         """Create a formatted STT transcription config.
@@ -602,11 +604,21 @@ class SpeechmaticsSTTService(STTService):
         Creates a transcription config object based on the service parameters. Aligns
         with the Speechmatics RT API transcription config.
         """
+        # Convert language if it's a Language enum
+        language = self._params.language
+        if isinstance(language, Language):
+            language = _language_to_speechmatics_language(language)
+
+        # Convert output locale if it's a Language enum
+        output_locale = self._params.output_locale
+        if isinstance(output_locale, Language):
+            output_locale = _locale_to_speechmatics_locale(language, output_locale)
+
         # Transcription config
         transcription_config = TranscriptionConfig(
-            language=self._params.language,
+            language=language,
             domain=self._params.domain,
-            output_locale=self._params.output_locale,
+            output_locale=output_locale,
             operating_point=self._params.operating_point,
             diarization="speaker" if self._params.enable_diarization else None,
             enable_partials=self._params.enable_partials,
@@ -618,7 +630,7 @@ class SpeechmaticsSTTService(STTService):
             transcription_config.additional_vocab = [
                 {
                     "content": e.content,
-                    "sounds_like": e.sounds_like,
+                    **({"sounds_like": e.sounds_like} if e.sounds_like else {}),
                 }
                 for e in self._params.additional_vocab
             ]
@@ -985,10 +997,10 @@ def _language_to_speechmatics_language(language: Language) -> str:
         language: The Language enum to convert.
 
     Returns:
-        str: The Speechmatics language code, if found.
+        str: The Speechmatics language code.
     """
     # List of supported input languages
-    BASE_LANGUAGES = {
+    LANGUAGE_MAP = {
         Language.AR: "ar",
         Language.BA: "ba",
         Language.EU: "eu",
@@ -1045,15 +1057,7 @@ def _language_to_speechmatics_language(language: Language) -> str:
         Language.CY: "cy",
     }
 
-    # Get the language code
-    result = BASE_LANGUAGES.get(language)
-
-    # Fail if language is not supported
-    if not result:
-        raise ValueError(f"Unsupported language: {language}")
-
-    # Return the language code
-    return result
+    return resolve_language(language, LANGUAGE_MAP, use_base_code=True)
 
 
 def _locale_to_speechmatics_locale(language_code: str, locale: Language) -> str | None:
